package net.carbon.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import lombok.extern.slf4j.Slf4j;
import net.carbon.enums.BizCodeEnum;
import net.carbon.interceptor.LoginInterceptor;
import net.carbon.mapper.UserMapper;
import net.carbon.model.LoginUser;
import net.carbon.model.po.UserDO;

import net.carbon.model.vo.ProjectVO;
import net.carbon.model.vo.UserVO;
import net.carbon.model.request.UserRegisterRequest;
import net.carbon.model.request.UserLoginRequest;
import net.carbon.service.UserService;
import net.carbon.utils.CheckUtil;
import net.carbon.utils.CommonUtil;
import net.carbon.utils.JWTUtil;
import net.carbon.utils.JsonData;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jieyuu
 * @since 2024-11-23
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, UserDO> implements UserService {

    @Autowired
    private UserMapper userMapper;

    @Override
    @Transactional(rollbackFor = RuntimeException.class, propagation = Propagation.REQUIRED)
    public JsonData register(UserRegisterRequest registerRequest) {
        UserDO userDO = new UserDO();

        BeanUtils.copyProperties(registerRequest, userDO);
        String cryptPwd = CommonUtil.MD5(userDO.getPassword());
        userDO.setPassword(cryptPwd);

        if (CheckUtil.isUsername(registerRequest.getUsername()) ||
                CheckUtil.isPassword(registerRequest.getPassword())) {

            //账号唯一性检查
            if (checkUnique(userDO.getUsername())) {
                //插入数据库
                int rows = userMapper.insert(userDO);
                log.info("rows:{},注册成功{}", rows, userDO.toString());

                return JsonData.buildSuccess();
            } else {
                return JsonData.buildResult(BizCodeEnum.ACCOUNT_REPEAT);
            }
        } else {
            return JsonData.buildResult(BizCodeEnum.ILLEGAL_ARGUMENT);
        }
    }


    /**
     * 校验用户账号唯一
     *
     * @param username
     * @return
     */
    private boolean checkUnique(String username) {
        QueryWrapper queryWrapper = new QueryWrapper<UserDO>().eq("username", username);

        List<UserDO> list = userMapper.selectList(queryWrapper);

        return list.size() > 0 ? false : true;
    }

    /**
     * 用户登陆注册
     * <p>
     * 判断用户注册情况
     *
     * @param loginRequest
     * @return
     */
    @Override
    public JsonData login(UserLoginRequest loginRequest) {
        LambdaQueryWrapper<UserDO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserDO::getUsername, loginRequest.getUsername());

        UserDO userDO = userMapper.selectOne(queryWrapper);

        if (userDO != null) {
            String cryptPwd = CommonUtil.MD5(loginRequest.getPassword());
            if (cryptPwd.equals(userDO.getPassword())) {
                //登陆成功
                //生成token令牌并且返回
                LoginUser loginUser = LoginUser.builder().build();
                BeanUtils.copyProperties(userDO, loginUser);
                //accessToken
                String accessToken = JWTUtil.geneJsonWebToken(loginUser);
                // accessToken过期时间
                //UUID生成一个token
                return JsonData.buildSuccess(accessToken);
            } else {
                return JsonData.buildResult(BizCodeEnum.ACCOUNT_PWD_ERROR);
            }

        } else {
            //未注册
            //这里选择 账号或密码错误，防止被爆破
            return JsonData.buildResult(BizCodeEnum.ACCOUNT_PWD_ERROR);
        }
    }

    @Override
    public UserVO userinfo() {

        LoginUser loginUser = LoginInterceptor.threadLocal.get();

        LambdaQueryWrapper<UserDO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserDO::getId, loginUser.getId());

        UserDO userDO = userMapper.selectOne(queryWrapper);
        UserVO userVO = new UserVO();
        BeanUtils.copyProperties(userDO, userVO);

        return userVO;
    }

    @Override
    public void updateUserinfo(UserDO userDO) {
        userDO.setUpdateTime(LocalDateTime.now());
        userMapper.updateUserinfo(userDO);
    }
}
